;;; -*- Mode: Lisp; Package: PVS -*-
(in-package "PVS")  ;; creates package for abstract syntax. 

(in-package "PVS")  ;; enters package for generated code.  

(use-package '("NEWATTR" "LANG" "SB-RUNTIME" "SORT" "TERM" "OCC" "OPER" "ERGOLISP"))


(export '())

(defparameter *PVS-OPSIG-TABLE*
    (make-opsig-table
     '((#^mappings
        . #@("pvs" := (#t(:list (:sort theory-decl-mapping))) :->
             #t(:sort theory-decl-mappings)))
       (#^names
        . #@("pvs" := (#t(:list (:sort name))) :-> #t(:sort names)))
       (#^eq . #@("pvs" := nil :-> #t(:sort lib-decl-opt-0)))
       (#^array . #@("pvs" := nil :-> #t(:sort fun-array)))
       (#^ceq . #@("pvs" := nil :-> #t(:sort assignment-alt-0)))
       (#^{\|\|} . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^[\|\|] . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^nomap . #@("pvs" := nil :-> #t(:sort modname-opt-1)))
       (#^comp-type-expr-NULL-2
        . #@("pvs" := (#t(:null)) :-> #t(:sort comp-type-expr-opt-2)))
       (#^funtype
        . #@("pvs" :=
             (#t(:sort comp-type-expr-opt-0)
              #t(:sort comp-type-expr-opt-1)
              #t(:sort comp-type-expr-opt-2))
             :-> #t(:sort comp-type-expr)))
       (#^expbuts
        . #@("pvs" := (#t(:list (:sort expname))) :->
             #t(:sort exportings-opt-0)))
       (#^& . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^conversion
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort conversion)))
       (#^lambda-body
        . #@("pvs" := (#t(:sort lambda-formals) #t(:sort expr)) :->
             #t(:sort lambda-body)))
       (#^pdf
        . #@("pvs" := (#t(:list (:sort pdformals))) :->
             #t(:sort theory-elt-opt-0)))
       (#^noexpr . #@("pvs" := nil :-> #t(:sort def-decl-opt-0)))
       (#^brace-vbar-expr
        . #@("pvs" := (#t(:list (:sort expr))) :-> #t(:sort expr)))
       (#^judgement-elt
        . #@("pvs" := (#t(:sort judgement) #t(:sort datatype-opt-2))
             :-> #t(:sort theory-elt)))
       (#^var-decl
        . #@("pvs" := (#t(:sort type-expr)) :-> #t(:sort var-decl)))
       (#^ftype-decl
        . #@("pvs" :=
             (#t(:sort type-keyword)
              #t(:sort theory-formal-decl-opt-0))
             :-> #t(:sort theory-formal-decl)))
       (#^<= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^auto-rewrite-elt
        . #@("pvs" := (#t(:sort auto-rewrite) #t(:sort datatype-opt-2))
             :-> #t(:sort theory-elt)))
       (#^bind
        . #@("pvs" := (#t(:sort bind-alt-0) #t(:sort expr)) :->
             #t(:sort bind)))
       (#^quant-type
        . #@("pvs" :=
             (#t(:sort type-expr-sans-name-alt-0)
              #t(:sort lambda-formals) #t(:sort type-expr))
             :-> #t(:sort type-expr-sans-name)))
       (#^conversion-elt
        . #@("pvs" := (#t(:sort conversion) #t(:sort datatype-opt-2))
             :-> #t(:sort theory-elt)))
       (#^not . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^typeplus . #@("pvs" := nil :-> #t(:sort type-keyword)))
       (#^has-type . #@("pvs" := nil :-> #t(:sort judgement-opt-0)))
       (#^noexportingmods
        . #@("pvs" := nil :-> #t(:sort exporting-opt-0)))
       (#^if-expr
        . #@("pvs" :=
             (#t(:sort expr) #t(:sort expr) #t(:sort elsifs)
              #t(:sort expr))
             :-> #t(:sort expr)))
       (#^assign-skoname
        . #@("pvs" := (#t(:sort sb-runtime::id) #t(:sort number)) :->
             #t(:sort assign-arg)))
       (#^assignment
        . #@("pvs" :=
             (#t(:sort args) #t(:sort expr) #t(:sort assignment-alt-0))
             :-> #t(:sort assignment)))
       (#^/= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^lemma . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^proposition . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^exporting-all
        . #@("pvs" := (#t(:sort exportings-opt-0)) :->
             #t(:sort exportings)))
       (#^theory-formal
        . #@("pvs" :=
             (#t(:sort theory-formal-opt-0) #t(:sort idopappls)
              #t(:sort theory-formal-decl))
             :-> #t(:sort theory-formal)))
       (#^list-expr
        . #@("pvs" := (#t(:list (:sort expr))) :-> #t(:sort expr)))
       (#^and . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^bind-expr
        . #@("pvs" := (#t(:sort expr-alt-0) #t(:sort lambda-body)) :->
             #t(:sort expr)))
       (#^expr-as-type
        . #@("pvs" := (#t(:sort expr)) :->
             #t(:sort type-expr-sans-name)))
       (#^rewrite-fnum
        . #@("pvs" :=
             (#t(:sort fnum) #t(:sort rewrite-name-or-fnum-opt-0)) :->
             #t(:sort rewrite-name-or-fnum)))
       (#^|##| . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^skovar
        . #@("pvs" := (#t(:sort sb-runtime::id) #t(:sort number)) :->
             #t(:sort expr)))
       (#^iff . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^args
        . #@("pvs" := (#t(:list (:sort assign-arg))) :->
             #t(:sort args)))
       (#^uninterp-const-decl
        . #@("pvs" := (#t(:sort type-expr)) :-> #t(:sort const-decl)))
       (#^jdecls
        . #@("pvs" := (#t(:list (:sort jdecl))) :-> #t(:sort jdecls)))
       (#^formula-decl
        . #@("pvs" := (#t(:sort formula-name) #t(:sort expr)) :->
             #t(:sort formula-decl)))
       (#^auto-rewrite
        . #@("pvs" := (#t(:list (:sort rewrite-name))) :->
             #t(:sort auto-rewrite)))
       (#^false . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^nocomma
        . #@("pvs" := nil :-> #t(:sort lambda-formals-opt-0)))
       (#^^^ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^>>= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^theory-decl
        . #@("pvs" := (#t(:sort theory-decl-modname)) :->
             #t(:sort theory-decl)))
       (#^exists
        . #@("pvs" := nil :-> #t(:sort type-expr-sans-name-alt-0)))
       (#^datatype
        . #@("pvs" :=
             (#t(:sort datatype-opt-0) #t(:sort datatype-opt-1)
              #t(:sort adtcases) #t(:sort sb-runtime::id))
             :-> #t(:sort datatype)))
       (#^expmodclosure . #@("pvs" := nil :-> #t(:sort exportingmods)))
       (#^importing
        . #@("pvs" := (#t(:list (:sort importing-item))) :->
             #t(:sort importing)))
       (#^fieldex
        . #@("pvs" := (#t(:sort sb-runtime::id) #t(:sort expr-opt-0))
             :-> #t(:sort expr)))
       (#^set-expr
        . #@("pvs" := (#t(:sort set-formals) #t(:sort expr)) :->
             #t(:sort set-expr)))
       (#^no-theory-formals
        . #@("pvs" := nil :-> #t(:sort adt-or-theory-opt-0)))
       (#^expname
        . #@("pvs" := (#t(:sort idop) #t(:sort expname-opt-0)) :->
             #t(:sort expname)))
       (#^fact . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^let-expr
        . #@("pvs" := (#t(:sort let-bindings) #t(:sort expr)) :->
             #t(:sort expr)))
       (#^tuple-expr
        . #@("pvs" := (#t(:list (:sort expr))) :-> #t(:sort expr)))
       (#^noelse . #@("pvs" := nil :-> #t(:sort expr-opt-2)))
       (#^table-entry
        . #@("pvs" :=
             (#t(:list
                 (:union (:op #^else) (:sort expr)
                  (:op #^notableentry))))
             :-> #t(:sort table-entry)))
       (#^assignments
        . #@("pvs" := (#t(:list (:sort assignment))) :->
             #t(:sort assignments)))
       (#^lambda-expr . #@("pvs" := nil :-> #t(:sort expr-alt-0)))
       (#^mapping-subst
        . #@("pvs" := (#t(:sort mapping-lhs) #t(:sort mapping-rhs)) :->
             #t(:sort theory-decl-mapping)))
       (#^theory-abbreviation-decl
        . #@("pvs" := (#t(:sort modname) #t(:sort sb-runtime::id)) :->
             #t(:sort importing-item)))
       (#^projex
        . #@("pvs" := (#t(:sort number) #t(:sort expr-opt-0)) :->
             #t(:sort expr)))
       (#^nomod . #@("pvs" := nil :-> #t(:sort name-opt-1)))
       (#^+ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^idops
        . #@("pvs" := (#t(:list (:sort idop))) :-> #t(:sort idops)))
       (#^type-decl
        . #@("pvs" :=
             (#t(:sort type-keyword) #t(:sort type-def)
              #t(:sort type-decl-opt-0))
             :-> #t(:sort type-decl)))
       (#^type-coercion
        . #@("pvs" := (#t(:sort type-expr)) :->
             #t(:sort type-coercion)))
       (#^corollary . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^binds
        . #@("pvs" := (#t(:list (:sort simplebind))) :->
             #t(:sort bind-alt-0)))
       (#^colrest
        . #@("pvs" := (#t(:list (:union (:sort expr) (:op #^else))))
             :-> #t(:sort colrest)))
       (#^> . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^table-entries
        . #@("pvs" := (#t(:list (:sort table-entry))) :->
             #t(:sort table-entries)))
       (#^modnames
        . #@("pvs" := (#t(:list (:sort modname))) :->
             #t(:sort modnames)))
       (#^term-expr
        . #@("pvs" := (#t(:sort |##|) #t(:sort expr-args)) :->
             #t(:sort expr)))
       (#^projappl
        . #@("pvs" := (#t(:sort expr) #t(:sort number)) :->
             #t(:sort expr)))
       (#^importing-elt
        . #@("pvs" := (#t(:sort importing) #t(:sort datatype-opt-2))
             :-> #t(:sort theory-elt)))
       (#^rec-expr
        . #@("pvs" := (#t(:sort assignments)) :-> #t(:sort expr)))
       (#^adformals
        . #@("pvs" := (#t(:list (:sort adformal))) :->
             #t(:sort adformals)))
       (#^idopappl
        . #@("pvs" := (#t(:sort idop) #t(:sort deptypes)) :->
             #t(:sort idopappl)))
       (#^mapping-rename
        . #@("pvs" := (#t(:sort mapping-lhs) #t(:sort mapping-rhs)) :->
             #t(:sort theory-decl-mapping)))
       (#^assumption
        . #@("pvs" := (#t(:sort expr)) :-> #t(:sort assumption)))
       (#^noformals . #@("pvs" := nil :-> #t(:sort theory-elt-opt-0)))
       (#^subtype
        . #@("pvs" := (#t(:sort set-formals) #t(:sort expr)) :->
             #t(:sort type-expr-sans-name)))
       (#^name-expr . #@("pvs" := (#t(:sort name)) :-> #t(:sort expr)))
       (#^adtdecls
        . #@("pvs" := (#t(:list (:op #^adtdecl))) :->
             #t(:sort constructor-opt-0)))
       (#^noacts . #@("pvs" := nil :-> #t(:sort name-opt-0)))
       (#^elsif
        . #@("pvs" := (#t(:sort expr) #t(:sort expr)) :->
             #t(:sort elsif)))
       (#^uninterp-type-decl
        . #@("pvs" := (#t(:sort type-keyword)) :->
             #t(:sort type-decl)))
       (#^actuals
        . #@("pvs" := (#t(:list (:sort actual))) :->
             #t(:sort actuals)))
       (#^forall-expr . #@("pvs" := nil :-> #t(:sort expr-alt-0)))
       (#^notgt . #@("pvs" := nil :-> #t(:sort modname-opt-2)))
       (#^idop
        . #@("pvs" := (#t(:sort opsym)) :-> #t(:sort idop-not-number)))
       (#^comma . #@("pvs" := nil :-> #t(:sort lambda-formals-opt-0)))
       (#^let-bindings
        . #@("pvs" := (#t(:list (:sort bind))) :->
             #t(:sort let-bindings)))
       (#^name-bind-expr
        . #@("pvs" := (#t(:sort sb-runtime::id) #t(:sort lambda-body))
             :-> #t(:sort expr)))
       (#^judgement
        . #@("pvs" :=
             (#t(:sort jdecls) #t(:sort judgement-opt-0)
              #t(:sort type-expr))
             :-> #t(:sort judgement)))
       (#^jsubtypedecl
        . #@("pvs" := (#t(:sort type-expr-sans-name)) :->
             #t(:sort jdecl)))
       (#^assign-tuple
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort assign-arg)))
       (#^paren-vbar-expr
        . #@("pvs" := (#t(:list (:sort expr))) :-> #t(:sort expr)))
       (#^nocontains . #@("pvs" := nil :-> #t(:sort type-decl-opt-0)))
       (#^codatatypes
        . #@("pvs" :=
             (#t(:sort subtypes) #t(:sort datatype-opt-0)
              #t(:sort datatype-opt-1) #t(:sort adtcases)
              #t(:sort sb-runtime::id))
             :-> #t(:sort datatype)))
       (#^expr-args
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort expr-args)))
       (#^expnames
        . #@("pvs" := (#t(:list (:sort expname))) :->
             #t(:sort exportings)))
       (#^typed-id
        . #@("pvs" :=
             (#t(:sort idop) #t(:sort typed-id-opt-0)
              #t(:sort typed-id-opt-1))
             :-> #t(:sort typed-id)))
       (#^col-heading
        . #@("pvs" := (#t(:sort expr) #t(:sort colrest)) :->
             #t(:sort col-heading)))
       (#^no-type-expr . #@("pvs" := nil :-> #t(:sort typed-id-opt-0)))
       (#^extended-type-name
        . #@("pvs" := (#t(:sort name) #t(:sort type-expr)) :->
             #t(:sort type-expr)))
       (#^fconst-decl
        . #@("pvs" := (#t(:sort type-expr)) :->
             #t(:sort theory-formal-decl)))
       (#^cond-case
        . #@("pvs" := (#t(:sort expr) #t(:sort expr)) :->
             #t(:sort cond-case)))
       (#^sublemma . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^def-decl
        . #@("pvs" :=
             (#t(:sort type-expr) #t(:sort expr) #t(:sort expr)
              #t(:sort def-decl-opt-0))
             :-> #t(:sort def-decl)))
       (#^conversionminus
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort conversion)))
       (#^adtcase
        . #@("pvs" := (#t(:sort constructor) #t(:sort idop)) :->
             #t(:sort adtcase)))
       (#^lib-fdecl
        . #@("pvs" := (#t(:sort idop) #t(:sort lib-decl)) :->
             #t(:sort theory-formal-opt-1)))
       (#^exptype . #@("pvs" := nil :-> #t(:sort expname-opt-0)))
       (#^set-list-expr
        . #@("pvs" := (#t(:list (:sort expr))) :-> #t(:sort set-expr)))
       (#^auto-rewriteminus
        . #@("pvs" := (#t(:list (:sort rewrite-name))) :->
             #t(:sort auto-rewrite)))
       (#^* . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^no-pred . #@("pvs" := nil :-> #t(:sort typed-id-opt-1)))
       (#^codatatype
        . #@("pvs" :=
             (#t(:sort datatype-opt-0) #t(:sort datatype-opt-1)
              #t(:sort adtcases) #t(:sort sb-runtime::id))
             :-> #t(:sort datatype)))
       (#^true . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^lambda-formals
        . #@("pvs" :=
             (#t(:sort lambda-formal) #t(:sort lambda-formals-opt-0)
              #t(:sort lambda-formals))
             :-> #t(:sort lambda-formals)))
       (#^= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^jnamedecl
        . #@("pvs" := (#t(:sort name)) :-> #t(:sort jdecl)))
       (#^const-decl
        . #@("pvs" := (#t(:sort type-expr) #t(:sort const-value)) :->
             #t(:sort const-decl)))
       (#^id-assign
        . #@("pvs" := (#t(:sort sb-runtime::id)) :->
             #t(:sort assign-arg)))
       (#^challenge . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^andthen . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^nosemic . #@("pvs" := nil :-> #t(:sort datatype-opt-2)))
       (#^noexp . #@("pvs" := nil :-> #t(:sort theory-opt-0)))
       (#^unary-term-expr
        . #@("pvs" := (#t(:sort -) #t(:sort expr)) :-> #t(:sort expr)))
       (#^\|- . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^module
        . #@("pvs" :=
             (#t(:sort theory-opt-0) #t(:sort theory-opt-1)
              #t(:sort theory-opt-2) #t(:sort sb-runtime::id))
             :-> #t(:sort theory)))
       (#^macro-decl
        . #@("pvs" := (#t(:sort type-expr) #t(:sort expr)) :->
             #t(:sort macro-decl)))
       (#^theorem . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^typed
        . #@("pvs" := (#t(:sort type-expr)) :->
             #t(:sort mapping-lhs-opt-0)))
       (#^|(\|\|)| . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^when . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^[] . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^fields
        . #@("pvs" := (#t(:list (:sort field-decls))) :->
             #t(:sort fields)))
       (#^elsifs
        . #@("pvs" := (#t(:list (:op #^elsif))) :-> #t(:sort elsifs)))
       (#^implies . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^<<= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^conversionplus
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort conversion)))
       (#^++ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^** . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^|#| . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^exporting
        . #@("pvs" := (#t(:sort exportings) #t(:sort exporting-opt-0))
             :-> #t(:sort exporting)))
       (#^proj-assign
        . #@("pvs" := (#t(:sort number)) :-> #t(:sort assign-arg)))
       (#^obligation . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^rec-has-type
        . #@("pvs" := nil :-> #t(:sort judgement-opt-0)))
       (#^recordtype
        . #@("pvs" := (#t(:sort fields)) :-> #t(:sort recordtype)))
       (#^cond-expr
        . #@("pvs" := (#t(:sort args) #t(:sort expr-opt-2)) :->
             #t(:sort expr)))
       (#^expr-NULL-1
        . #@("pvs" := (#t(:null)) :-> #t(:sort expr-opt-1)))
       (#^<\| . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^equal . #@("pvs" := nil :-> #t(:sort type-def-alt-0)))
       (#^deptypes
        . #@("pvs" := (#t(:list (:sort type-expr))) :->
             #t(:sort deptypes)))
       (#^type-name
        . #@("pvs" := (#t(:sort name)) :-> #t(:sort type-expr)))
       (#^arguments
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort arguments)))
       (#^assign-num
        . #@("pvs" := (#t(:sort number)) :-> #t(:sort assign-arg)))
       (#^struct-subtype
        . #@("pvs" := (#t(:sort type-expr)) :->
             #t(:sort theory-formal-decl-opt-0)))
       (#^emptytuple
        . #@("pvs" := nil :-> #t(:sort comp-type-expr-opt-1)))
       (#^noass . #@("pvs" := nil :-> #t(:sort theory-opt-1)))
       (#^theory-formals
        . #@("pvs" := (#t(:list (:sort theory-formal))) :->
             #t(:sort theory-formals)))
       (#^o . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^noeq . #@("pvs" := nil :-> #t(:sort lib-decl-opt-0)))
       (#^else . #@("pvs" := nil :-> #t(:sort table-entry-opt-0)))
       (#^subtypes
        . #@("pvs" := (#t(:list (:sort sb-runtime::id))) :->
             #t(:sort subtypes)))
       (#^semic . #@("pvs" := nil :-> #t(:sort datatype-opt-2)))
       (#^/\\ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^< . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^noactuals . #@("pvs" := nil :-> #t(:sort expr-opt-0)))
       (#^ename
        . #@("pvs" := (#t(:sort name) #t(:sort ename-opt-0)) :->
             #t(:sort ename)))
       (#^rewrite-name
        . #@("pvs" :=
             (#t(:sort name) #t(:sort rewrite-name-or-fnum-opt-0)
              #t(:sort rewrite-name-opt-0))
             :-> #t(:sort rewrite-name)))
       (#^>> . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^bname
        . #@("pvs" := (#t(:sort name) #t(:sort number)) :->
             #t(:sort bname)))
       (#^noexpkind . #@("pvs" := nil :-> #t(:sort expname-opt-0)))
       (#^|<:| . #@("pvs" := nil :-> #t(:sort type-def-alt-0)))
       (#^or . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^adt-or-modules
        . #@("pvs" := (#t(:list (:sort adt-or-theory))) :->
             #t(:sort adt-or-theories)))
       (#^assign-id
        . #@("pvs" := (#t(:sort sb-runtime::id)) :->
             #t(:sort assign-arg)))
       (#^fun-arguments
        . #@("pvs" := (#t(:list (:sort expr))) :->
             #t(:sort fun-arguments)))
       (#^norowvar . #@("pvs" := nil :-> #t(:sort table-expr-opt-0)))
       (#^if . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^xor . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^/ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^formula . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^axiom . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^corec-decl
        . #@("pvs" := (#t(:sort type-expr) #t(:sort expr)) :->
             #t(:sort corec-decl)))
       (#^notype . #@("pvs" := nil :-> #t(:sort ename-opt-0)))
       (#^fieldappl
        . #@("pvs" := (#t(:sort expr) #t(:sort sb-runtime::id)) :->
             #t(:sort expr)))
       (#^forall
        . #@("pvs" := nil :-> #t(:sort type-expr-sans-name-alt-0)))
       (#^dep-binding
        . #@("pvs" := (#t(:sort idop) #t(:sort type-expr)) :->
             #t(:sort dep-type-expr)))
       (#^notheory . #@("pvs" := nil :-> #t(:sort theory-opt-2)))
       (#^adt-or-module
        . #@("pvs" :=
             (#t(:sort sb-runtime::id) #t(:sort adt-or-theory-opt-0)
              #t(:sort adt-or-theory-alt-0))
             :-> #t(:sort adt-or-theory)))
       (#^law . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^unique-name
        . #@("pvs" := (#t(:sort name) #t(:sort unique-name-opt-0)) :->
             #t(:sort unique-name)))
       (#^extended-type-appl
        . #@("pvs" :=
             (#t(:sort name) #t(:sort arguments) #t(:sort type-expr))
             :-> #t(:sort type-expr)))
       (#^nonempty-type . #@("pvs" := nil :-> #t(:sort type-keyword)))
       (#^macro
        . #@("pvs" := nil :-> #t(:sort rewrite-name-or-fnum-opt-0)))
       (#^mapping-def
        . #@("pvs" := (#t(:sort mapping-lhs) #t(:sort mapping-rhs)) :->
             #t(:sort theory-decl-mapping)))
       (#^dom
        . #@("pvs" := (#t(:list (:sort dep-type-expr))) :->
             #t(:sort dep-type-exprs)))
       (#^ind-decl
        . #@("pvs" := (#t(:sort type-expr) #t(:sort expr)) :->
             #t(:sort ind-decl)))
       (#^datatypes
        . #@("pvs" :=
             (#t(:sort subtypes) #t(:sort datatype-opt-0)
              #t(:sort datatype-opt-1) #t(:sort adtcases)
              #t(:sort sb-runtime::id))
             :-> #t(:sort datatype)))
       (#^type-appl
        . #@("pvs" := (#t(:sort name) #t(:sort arguments)) :->
             #t(:sort type-expr)))
       (#^coercion
        . #@("pvs" := (#t(:sort expr) #t(:sort type-coercion)) :->
             #t(:sort expr)))
       (#^\|= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^nocolvar . #@("pvs" := nil :-> #t(:sort table-expr-opt-1)))
       (#^eager
        . #@("pvs" := nil :-> #t(:sort rewrite-name-or-fnum-opt-0)))
       (#^name
        . #@("pvs" :=
             (#t(:sort idop) #t(:sort modname-opt-0)
              #t(:sort name-opt-0) #t(:sort name-opt-1)
              #t(:sort modname-opt-1) #t(:sort modname-opt-2))
             :-> #t(:sort name)))
       (#^field-decls
        . #@("pvs" := (#t(:sort ids) #t(:sort type-expr)) :->
             #t(:sort field-decls)))
       (#^arr . #@("pvs" := nil :-> #t(:sort assignment-alt-0)))
       (#^table-expr
        . #@("pvs" :=
             (#t(:sort table-expr-opt-0) #t(:sort table-expr-opt-1)
              #t(:sort table-expr-opt-2) #t(:sort table-entries))
             :-> #t(:sort table-expr)))
       (#^brack-expr
        . #@("pvs" := (#t(:list (:sort expr))) :-> #t(:sort expr)))
       (#^set-formals
        . #@("pvs" :=
             (#t(:sort set-formal) #t(:sort lambda-formals-opt-0)
              #t(:sort set-formals))
             :-> #t(:sort set-formals)))
       (#^where-expr
        . #@("pvs" := (#t(:sort let-bindings) #t(:sort expr)) :->
             #t(:sort expr)))
       (#^varianttype
        . #@("pvs" := (#t(:sort fields)) :-> #t(:sort recordtype)))
       (#^noqual . #@("pvs" := nil :-> #t(:sort mapping-lhs-opt-0)))
       (#^@@ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^idopappls
        . #@("pvs" := (#t(:list (:sort idopappl))) :->
             #t(:sort idopappls)))
       (#^theory-part
        . #@("pvs" := (#t(:list (:sort theory-elt))) :->
             #t(:sort theory-part)))
       (#^function . #@("pvs" := nil :-> #t(:sort fun-array)))
       (#^=> . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^>= . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^datatype-NULL-2
        . #@("pvs" := (#t(:null)) :-> #t(:sort datatype-opt-0)))
       (#^expmodall . #@("pvs" := nil :-> #t(:sort exportingmods)))
       (#^\\/ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^modname
        . #@("pvs" :=
             (#t(:sort sb-runtime::id) #t(:sort modname-opt-0)
              #t(:sort expr-opt-0) #t(:sort theory-decl-modname-opt-0)
              #t(:sort theory-decl-modname-opt-1))
             :-> #t(:sort theory-decl-modname)))
       (#^auto-rewriteplus
        . #@("pvs" := (#t(:list (:sort rewrite-name))) :->
             #t(:sort auto-rewrite)))
       (#^assuming-part
        . #@("pvs" := (#t(:list (:sort assuming))) :->
             #t(:sort assumings)))
       (#^theory . #@("pvs" := nil :-> #t(:sort mapping-lhs-opt-0)))
       (#^noexpbuts . #@("pvs" := nil :-> #t(:sort exportings-opt-0)))
       (#^extended-type
        . #@("pvs" :=
             (#t(:sort type-expr-sans-name) #t(:sort type-expr)) :->
             #t(:sort type-expr)))
       (#^type . #@("pvs" := nil :-> #t(:sort mapping-lhs-opt-0)))
       (#^application
        . #@("pvs" := (#t(:sort expr) #t(:sort fun-arguments)) :->
             #t(:sort expr)))
       (#^lazy
        . #@("pvs" := nil :-> #t(:sort rewrite-name-or-fnum-opt-0)))
       (#^rec-subtype-of
        . #@("pvs" := nil :-> #t(:sort judgement-opt-0)))
       (#^ids
        . #@("pvs" := (#t(:list (:sort sb-runtime::id))) :->
             #t(:sort ids)))
       (#^exists-expr . #@("pvs" := nil :-> #t(:sort expr-alt-0)))
       (#^datatype-NULL-1
        . #@("pvs" := (#t(:null)) :-> #t(:sort datatype-opt-1)))
       (#^assuming
        . #@("pvs" :=
             (#t(:sort idops) #t(:sort pdf) #t(:sort assuming-decl)
              #t(:sort datatype-opt-2))
             :-> #t(:sort assuming)))
       (#^from . #@("pvs" := nil :-> #t(:sort type-def-alt-0)))
       (#^theory-formal-NULL-1
        . #@("pvs" := (#t(:null)) :-> #t(:sort theory-formal-opt-0)))
       (#^adtdecl
        . #@("pvs" := (#t(:sort idops) #t(:sort type-expr)) :->
             #t(:sort adtdecl)))
       (#^\|> . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^// . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^constructor
        . #@("pvs" := (#t(:sort idop) #t(:sort constructor-opt-0)) :->
             #t(:sort constructor)))
       (#^selection-NULL-1
        . #@("pvs" := (#t(:null)) :-> #t(:sort selection-opt-0)))
       (#^mapping-lhs
        . #@("pvs" :=
             (#t(:sort idop) #t(:sort theory-elt-opt-0)
              #t(:sort mapping-lhs-opt-0))
             :-> #t(:sort mapping-lhs)))
       (#^nocolheading
        . #@("pvs" := nil :-> #t(:sort table-expr-opt-2)))
       (#^formal-importing-elt
        . #@("pvs" :=
             (#t(:sort theory-formal-opt-1) #t(:sort importing)) :->
             #t(:sort theory-formal-opt-0)))
       (#^orelse . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^expformula . #@("pvs" := nil :-> #t(:sort expname-opt-0)))
       (#^enumtype
        . #@("pvs" := (#t(:sort set-formals)) :->
             #t(:sort type-expr-sans-name)))
       (#^cases-expr
        . #@("pvs" :=
             (#t(:sort expr) #t(:sort args) #t(:sort expr-opt-1)) :->
             #t(:sort expr)))
       (#^update-expr
        . #@("pvs" := (#t(:sort expr) #t(:sort assignments)) :->
             #t(:sort expr)))
       (#^string-expr
        . #@("pvs" := (#t(:sort string)) :-> #t(:sort expr)))
       (#^fnum
        . #@("pvs" := (#t(:sort number) #t(:sort -)) :->
             #t(:sort fnum)))
       (#^conjecture . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^~ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^^ . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^<> . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^postulate . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^== . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^<< . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^adtcases
        . #@("pvs" := (#t(:list (:sort adtcase))) :->
             #t(:sort adtcases)))
       (#^constructor-NULL-1
        . #@("pvs" := (#t(:null)) :-> #t(:sort constructor-opt-0)))
       (#^- . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^<=> . #@("pvs" := nil :-> #t(:sort opsym)))
       (#^cotupletype
        . #@("pvs" := (#t(:list (:sort dep-type-expr))) :->
             #t(:sort dep-type-exprs)))
       (#^jappldecl
        . #@("pvs" := (#t(:sort name) #t(:sort pdf)) :->
             #t(:sort jdecl)))
       (#^lib-decl
        . #@("pvs" := (#t(:sort lib-decl-opt-0) #t(:sort string)) :->
             #t(:sort lib-decl)))
       (#^coind-decl
        . #@("pvs" := (#t(:sort type-expr) #t(:sort expr)) :->
             #t(:sort coind-decl)))
       (#^comp-type-expr-NULL-1
        . #@("pvs" := (#t(:null)) :-> #t(:sort comp-type-expr-opt-0)))
       (#^ftheory-decl
        . #@("pvs" := (#t(:sort theory-decl-modname)) :->
             #t(:sort theory-formal-decl)))
       (#^subtype-of . #@("pvs" := nil :-> #t(:sort judgement-opt-0)))
       (#^simplebind
        . #@("pvs" :=
             (#t(:sort idop) #t(:sort pdf) #t(:sort ename-opt-0)) :->
             #t(:sort simplebind)))
       (#^type-def
        . #@("pvs" := (#t(:sort type-def-alt-0) #t(:sort type-expr))
             :-> #t(:sort type-def)))
       (#^claim . #@("pvs" := nil :-> #t(:sort formula-name)))
       (#^typed-ids
        . #@("pvs" :=
             (#t(:sort idops) #t(:sort typed-id-opt-0)
              #t(:sort typed-id-opt-1))
             :-> #t(:sort typed-ids)))
       (#^set-id
        . #@("pvs" := (#t(:sort idop) #t(:sort ename-opt-0)) :->
             #t(:sort set-formal)))
       (#^selection
        . #@("pvs" :=
             (#t(:sort idop) #t(:sort selection-opt-0) #t(:sort expr))
             :-> #t(:sort selection)))
       (#^adtcase-subtype
        . #@("pvs" :=
             (#t(:sort constructor) #t(:sort idop)
              #t(:sort sb-runtime::id))
             :-> #t(:sort adtcase)))
       (#^notableentry
        . #@("pvs" := nil :-> #t(:sort table-entry-opt-0)))
       (#^nolib . #@("pvs" := nil :-> #t(:sort modname-opt-0))))))


(defparameter *PVS-SORT-TABLE*
    (make-sort-table
     '((#t(:sort mappings) . #t(:op #^mappings))
       (#t(:sort names) . #t(:op #^names))
       (#t(:sort set-formal)
        . #t(:union (:sort pdformals) (:op #^set-id)))
       (#t(:sort table-entry-opt-0)
        . #t(:union (:op #^else) (:sort expr) (:op #^notableentry)))
       (#t(:sort comp-type-expr-opt-2)
        . #t(:union (:sort type-expr) (:op #^comp-type-expr-NULL-2)))
       (#t(:sort rewrite-name-opt-0)
        . #t(:union (:op #^formula) (:op #^type) (:op #^noqual)))
       (#t(:sort &) . #t(:op #^&))
       (#t(:sort conversion)
        . #t(:union (:op #^conversionminus) (:op #^conversionplus)
             (:op #^conversion)))
       (#t(:sort lambda-body) . #t(:op #^lambda-body))
       (#t(:sort pdf) . #t(:op #^pdf))
       (#t(:sort var-decl) . #t(:op #^var-decl))
       (#t(:sort <=) . #t(:op #^<=))
       (#t(:sort comp-type-expr-opt-1)
        . #t(:union (:sort dep-type-exprs) (:op #^emptytuple)))
       (#t(:sort modname-opt-0)
        . #t(:union (:sort sb-runtime::id) (:op #^nolib)))
       (#t(:sort bind) . #t(:op #^bind))
       (#t(:sort exportings-opt-0)
        . #t(:union (:op #^expbuts) (:op #^noexpbuts)))
       (#t(:sort not) . #t(:op #^not))
       (#t(:sort exportings)
        . #t(:union (:op #^expnames) (:op #^exporting-all)))
       (#t(:sort theory-declaration)
        . #t(:union (:sort judgement) (:sort datatype)
             (:sort formula-decl) (:sort coind-decl) (:sort corec-decl)
             (:sort ind-decl) (:sort def-decl) (:sort macro-decl)
             (:sort const-decl) (:sort var-decl) (:sort type-decl)
             (:sort theory-decl) (:sort lib-decl)))
       (#t(:sort dep-type-exprs)
        . #t(:union (:op #^cotupletype) (:op #^dom)))
       (#t(:sort /=) . #t(:op #^/=))
       (#t(:sort assignment) . #t(:op #^assignment))
       (#t(:sort theory-formal) . #t(:op #^theory-formal))
       (#t(:sort and) . #t(:op #^and))
       (#t(:sort |##|) . #t(:op #^|##|))
       (#t(:sort iff) . #t(:op #^iff))
       (#t(:sort args) . #t(:op #^args))
       (#t(:sort jdecls) . #t(:op #^jdecls))
       (#t(:sort formula-decl) . #t(:op #^formula-decl))
       (#t(:sort auto-rewrite)
        . #t(:union (:op #^auto-rewriteminus) (:op #^auto-rewriteplus)
             (:op #^auto-rewrite)))
       (#t(:sort type-expr)
        . #t(:union (:op #^extended-type) (:sort type-expr-sans-name)
             (:op #^extended-type-appl) (:op #^extended-type-name)
             (:op #^type-appl) (:op #^type-name)))
       (#t(:sort formula-name)
        . #t(:union (:op #^theorem) (:op #^sublemma)
             (:op #^proposition) (:op #^postulate) (:op #^obligation)
             (:op #^lemma) (:op #^law) (:op #^formula) (:op #^fact)
             (:op #^corollary) (:op #^conjecture) (:op #^claim)
             (:op #^challenge) (:op #^axiom)))
       (#t(:sort ^^) . #t(:op #^^^)) (#t(:sort >>=) . #t(:op #^>>=))
       (#t(:sort theory-decl) . #t(:op #^theory-decl))
       (#t(:sort datatype)
        . #t(:union (:op #^codatatypes) (:op #^codatatype)
             (:op #^datatypes) (:op #^datatype)))
       (#t(:sort assignment-alt-0)
        . #t(:union (:op #^arr) (:op #^ceq)))
       (#t(:sort importing) . #t(:op #^importing))
       (#t(:sort set-expr)
        . #t(:union (:op #^set-list-expr) (:op #^set-expr)))
       (#t(:sort expname) . #t(:op #^expname))
       (#t(:sort table-entry) . #t(:op #^table-entry))
       (#t(:sort assignments) . #t(:op #^assignments))
       (#t(:sort theory-elt-opt-0)
        . #t(:union (:op #^pdf) (:op #^noformals)))
       (#t(:sort +) . #t(:op #^+)) (#t(:sort idops) . #t(:op #^idops))
       (#t(:sort type-decl)
        . #t(:union (:op #^type-decl) (:op #^uninterp-type-decl)))
       (#t(:sort mapping-rhs)
        . #t(:union (:sort comp-type-expr) (:sort expr)))
       (#t(:sort type-coercion) . #t(:op #^type-coercion))
       (#t(:sort theory-formal-decl)
        . #t(:union (:op #^ftheory-decl) (:op #^fconst-decl)
             (:op #^ftype-decl)))
       (#t(:sort colrest) . #t(:op #^colrest))
       (#t(:sort >) . #t(:op #^>))
       (#t(:sort table-entries) . #t(:op #^table-entries))
       (#t(:sort pdformals) . #t(:sort adformals))
       (#t(:sort modnames) . #t(:op #^modnames))
       (#t(:sort expr)
        . #t(:union (:sort table-expr) (:op #^cond-expr)
             (:op #^cases-expr) (:op #^update-expr) (:op #^where-expr)
             (:op #^let-expr) (:sort set-expr) (:op #^name-bind-expr)
             (:op #^skovar) (:op #^bind-expr) (:op #^if-expr)
             (:op #^coercion) (:op #^unary-term-expr) (:op #^term-expr)
             (:op #^application) (:op #^projappl) (:op #^fieldappl)
             (:op #^projex) (:op #^fieldex) (:op #^tuple-expr)
             (:op #^rec-expr) (:op #^brace-vbar-expr)
             (:op #^paren-vbar-expr) (:op #^brack-expr)
             (:op #^list-expr) (:op #^name-expr) (:op #^string-expr)))
       (#t(:sort theory-decl-modname-opt-1)
        . #t(:union (:sort theory-decl-modname) (:op #^notgt)))
       (#t(:sort adformals) . #t(:op #^adformals))
       (#t(:sort idopappl) . #t(:union (:op #^idopappl) (:sort idop)))
       (#t(:sort expr-opt-1)
        . #t(:union (:sort expr) (:op #^expr-NULL-1)))
       (#t(:sort assumption) . #t(:op #^assumption))
       (#t(:sort theory-opt-1)
        . #t(:union (:sort assuming-part) (:op #^noass)))
       (#t(:sort elsif) . #t(:op #^elsif))
       (#t(:sort actuals) . #t(:op #^actuals))
       (#t(:sort idop) . #t(:op #^idop))
       (#t(:sort let-bindings) . #t(:op #^let-bindings))
       (#t(:sort judgement) . #t(:op #^judgement))
       (#t(:sort expr-args) . #t(:op #^expr-args))
       (#t(:sort typed-id) . #t(:op #^typed-id))
       (#t(:sort adt-or-theory) . #t(:op #^adt-or-module))
       (#t(:sort theory-formal-opt-1)
        . #t(:union (:op #^lib-fdecl) (:op #^nolib)))
       (#t(:sort table-expr-opt-1)
        . #t(:union (:sort expr) (:op #^nocolvar)))
       (#t(:sort theory-formal-opt-0)
        . #t(:union (:op #^formal-importing-elt)
             (:op #^theory-formal-NULL-1)))
       (#t(:sort adt-or-theory-alt-0)
        . #t(:union (:sort datatype) (:sort theory)))
       (#t(:sort col-heading) . #t(:op #^col-heading))
       (#t(:sort cond-case) . #t(:op #^cond-case))
       (#t(:sort def-decl) . #t(:op #^def-decl))
       (#t(:sort adtcase)
        . #t(:union (:op #^adtcase-subtype) (:op #^adtcase)))
       (#t(:sort *) . #t(:op #^*)) (#t(:sort =) . #t(:op #^=))
       (#t(:sort lambda-formals)
        . #t(:union (:op #^lambda-formals) (:sort lambda-formal)))
       (#t(:sort const-decl)
        . #t(:union (:op #^const-decl) (:op #^uninterp-const-decl)))
       (#t(:sort andthen) . #t(:op #^andthen))
       (#t(:sort \|-) . #t(:op #^\|-))
       (#t(:sort typed-id-opt-0)
        . #t(:union (:sort type-expr) (:op #^no-type-expr)))
       (#t(:sort macro-decl) . #t(:op #^macro-decl))
       (#t(:sort opsym)
        . #t(:union (:op #^{\|\|}) (:op #^|(\|\|)|) (:op #^[\|\|])
             (:op #^\|>) (:op #^<\|) (:op #^\|=) (:op #^\|-)
             (:op #^|##|) (:op #^@@) (:op #^|#|) (:op #^>>=)
             (:op #^<<=) (:op #^>>) (:op #^<<) (:op #^^^) (:op #^//)
             (:op #^**) (:op #^++) (:op #^andthen) (:op #^orelse)
             (:op #^xor) (:op #^o) (:op #^false) (:op #^true)
             (:op #^if) (:op #^<=>) (:op #^=>) (:op #^~) (:op #^<>)
             (:op #^[]) (:op #^^) (:op #^\\/) (:op #^/\\) (:op #^&)
             (:op #^not) (:op #^and) (:op #^or) (:op #^when)
             (:op #^implies) (:op #^iff) (:op #^>=) (:op #^>)
             (:op #^<=) (:op #^<) (:op #^==) (:op #^/=) (:op #^=)
             (:op #^/) (:op #^*) (:op #^-) (:op #^+)))
       (#t(:sort lambda-formal)
        . #t(:union (:sort pdformals) (:sort idop-not-number)))
       (#t(:sort lambda-formals-opt-0)
        . #t(:union (:op #^comma) (:op #^nocomma)))
       (#t(:sort fields) . #t(:op #^fields))
       (#t(:sort when) . #t(:op #^when)) (#t(:sort []) . #t(:op #^[]))
       (#t(:sort exportingmods)
        . #t(:union (:sort modnames) (:op #^expmodclosure)
             (:op #^expmodall)))
       (#t(:sort elsifs) . #t(:op #^elsifs))
       (#t(:sort implies) . #t(:op #^implies))
       (#t(:sort <<=) . #t(:op #^<<=)) (#t(:sort **) . #t(:op #^**))
       (#t(:sort ++) . #t(:op #^++))
       (#t(:sort selection-opt-0)
        . #t(:union (:sort idops) (:op #^selection-NULL-1)))
       (#t(:sort adt-or-theory-opt-0)
        . #t(:union (:sort theory-formals) (:op #^no-theory-formals)))
       (#t(:sort |#|) . #t(:op #^|#|))
       (#t(:sort exporting) . #t(:op #^exporting))
       (#t(:sort theory-decl-modname-opt-0)
        . #t(:union (:sort theory-decl-mappings) (:op #^nomap)))
       (#t(:sort expr-opt-0)
        . #t(:union (:sort actuals) (:op #^noactuals)))
       (#t(:sort def-decl-opt-0)
        . #t(:union (:sort expr) (:op #^noexpr)))
       (#t(:sort comp-type-expr)
        . #t(:union (:op #^funtype) (:sort recordtype)))
       (#t(:sort datatype-opt-0)
        . #t(:union (:sort assuming-part) (:op #^datatype-NULL-2)))
       (#t(:sort recordtype)
        . #t(:union (:op #^varianttype) (:op #^recordtype)))
       (#t(:sort <\|) . #t(:op #^<\|))
       (#t(:sort deptypes) . #t(:op #^deptypes))
       (#t(:sort arguments) . #t(:op #^arguments))
       (#t(:sort expr-alt-0)
        . #t(:union (:op #^exists-expr) (:op #^forall-expr)
             (:op #^lambda-expr)))
       (#t(:sort type-def-alt-0)
        . #t(:union (:op #^|<:|) (:op #^from) (:op #^equal)))
       (#t(:sort theory-formals) . #t(:op #^theory-formals))
       (#t(:sort mapping-lhs-opt-0)
        . #t(:union (:op #^typed) (:op #^theory) (:op #^type)
             (:op #^noqual)))
       (#t(:sort o) . #t(:op #^o))
       (#t(:sort adformal)
        . #t(:union (:sort typed-ids) (:sort typed-id)))
       (#t(:sort subtypes) . #t(:op #^subtypes))
       (#t(:sort rewrite-name-or-fnum)
        . #t(:union (:sort rewrite-name) (:op #^rewrite-fnum)))
       (#t(:sort modname-opt-1)
        . #t(:union (:sort mappings) (:op #^nomap)))
       (#t(:sort /\\) . #t(:op #^/\\)) (#t(:sort <) . #t(:op #^<))
       (#t(:sort >>) . #t(:op #^>>))
       (#t(:sort ename) . #t(:op #^ename))
       (#t(:sort rewrite-name) . #t(:op #^rewrite-name))
       (#t(:sort modname-opt-2)
        . #t(:union (:sort modname) (:op #^notgt)))
       (#t(:sort bname) . #t(:union (:op #^bname) (:sort name)))
       (#t(:sort or) . #t(:op #^or))
       (#t(:sort assign-arg)
        . #t(:union (:op #^assign-num) (:op #^assign-skoname)
             (:op #^assign-id) (:op #^proj-assign) (:op #^id-assign)
             (:op #^assign-tuple)))
       (#t(:sort fun-arguments) . #t(:op #^fun-arguments))
       (#t(:sort constructor-opt-0)
        . #t(:union (:op #^adtdecls) (:op #^constructor-NULL-1)))
       (#t(:sort xor) . #t(:op #^xor)) (#t(:sort /) . #t(:op #^/))
       (#t(:sort corec-decl) . #t(:op #^corec-decl))
       (#t(:sort theory-decl-modname) . #t(:op #^modname))
       (#t(:sort theory-elt)
        . #t(:union (:op #^theory) (:op #^auto-rewrite-elt)
             (:op #^conversion-elt) (:op #^judgement-elt)
             (:op #^importing-elt)))
       (#t(:sort unique-name) . #t(:op #^unique-name))
       (#t(:sort ind-decl) . #t(:op #^ind-decl))
       (#t(:sort idop-not-number) . #t(:op #^idop))
       (#t(:sort \|=) . #t(:op #^\|=))
       (#t(:sort name-opt-0)
        . #t(:union (:sort actuals) (:op #^noacts)))
       (#t(:sort name) . #t(:op #^name))
       (#t(:sort field-decls) . #t(:op #^field-decls))
       (#t(:sort datatype-opt-2)
        . #t(:union (:op #^semic) (:op #^nosemic)))
       (#t(:sort table-expr) . #t(:op #^table-expr))
       (#t(:sort type-decl-opt-0)
        . #t(:union (:sort expr) (:op #^nocontains)))
       (#t(:sort set-formals)
        . #t(:union (:op #^set-formals) (:sort set-formal)))
       (#t(:sort @@) . #t(:op #^@@))
       (#t(:sort ename-opt-0)
        . #t(:union (:sort type-expr) (:op #^notype)))
       (#t(:sort theory-formal-decl-opt-0)
        . #t(:union (:op #^struct-subtype) (:sort type-expr)
             (:op #^notype)))
       (#t(:sort idopappls) . #t(:op #^idopappls))
       (#t(:sort theory-part) . #t(:op #^theory-part))
       (#t(:sort =>) . #t(:op #^=>))
       (#t(:sort exporting-opt-0)
        . #t(:union (:sort exportingmods) (:op #^noexportingmods)))
       (#t(:sort type-expr-sans-name-alt-0)
        . #t(:union (:op #^exists) (:op #^forall)))
       (#t(:sort >=) . #t(:op #^>=))
       (#t(:sort dep-type-expr)
        . #t(:union (:op #^dep-binding) (:sort type-expr)))
       (#t(:sort typed-id-opt-1)
        . #t(:union (:sort expr) (:op #^no-pred)))
       (#t(:sort lib-decl-opt-0) . #t(:union (:op #^eq) (:op #^noeq)))
       (#t(:sort table-expr-opt-0)
        . #t(:union (:sort expr) (:op #^norowvar)))
       (#t(:sort \\/) . #t(:op #^\\/))
       (#t(:sort modname) . #t(:op #^modname))
       (#t(:sort actual)
        . #t(:union (:sort mappings) (:sort comp-type-expr)
             (:sort expr)))
       (#t(:sort mapping) . #t(:op #^mapping-subst))
       (#t(:sort const-value) . #t(:sort expr))
       (#t(:sort comp-type-expr-opt-0)
        . #t(:union (:sort fun-array) (:op #^comp-type-expr-NULL-1)))
       (#t(:sort assuming-part) . #t(:sort assumings))
       (#t(:sort theory) . #t(:op #^module))
       (#t(:sort expr-opt-2) . #t(:union (:sort expr) (:op #^noelse)))
       (#t(:sort type-expr-sans-name)
        . #t(:union (:op #^quant-type) (:sort comp-type-expr)
             (:op #^subtype) (:op #^enumtype) (:op #^expr-as-type)))
       (#t(:sort assuming-decl)
        . #t(:union (:sort assumption) (:sort theory-declaration)))
       (#t(:sort ids) . #t(:op #^ids))
       (#t(:sort assuming)
        . #t(:union (:op #^assuming) (:op #^auto-rewrite-elt)
             (:op #^conversion-elt) (:op #^judgement-elt)
             (:op #^importing-elt)))
       (#t(:sort datatype-opt-1)
        . #t(:union (:op #^importing-elt) (:op #^datatype-NULL-1)))
       (#t(:sort col-heading-alt-0)
        . #t(:union (:op #^else) (:sort expr)))
       (#t(:sort importing-item)
        . #t(:union (:op #^theory-abbreviation-decl) (:sort modname)))
       (#t(:sort table-expr-opt-2)
        . #t(:union (:sort col-heading) (:op #^nocolheading)))
       (#t(:sort theory-opt-0)
        . #t(:union (:sort exporting) (:op #^noexp)))
       (#t(:sort adtdecl) . #t(:op #^adtdecl))
       (#t(:sort \|>) . #t(:op #^\|>))
       (#t(:sort theory-decl-mappings) . #t(:op #^mappings))
       (#t(:sort //) . #t(:op #^//))
       (#t(:sort constructor) . #t(:op #^constructor))
       (#t(:sort theory-decl-mapping)
        . #t(:union (:op #^mapping-rename) (:op #^mapping-subst)
             (:op #^mapping-def)))
       (#t(:sort mapping-lhs) . #t(:op #^mapping-lhs))
       (#t(:sort orelse) . #t(:op #^orelse))
       (#t(:sort theory-opt-2)
        . #t(:union (:sort theory-part) (:op #^notheory)))
       (#t(:sort jdecl)
        . #t(:union (:op #^jappldecl) (:op #^jnamedecl)
             (:op #^jsubtypedecl)))
       (#t(:sort name-opt-1) . #t(:union (:sort idop) (:op #^nomod)))
       (#t(:sort assumings) . #t(:op #^assuming-part))
       (#t(:sort fnum) . #t(:op #^fnum)) (#t(:sort ~) . #t(:op #^~))
       (#t(:sort ^) . #t(:op #^^))
       (#t(:sort rewrite-name-or-fnum-opt-0)
        . #t(:union (:op #^macro) (:op #^eager) (:op #^lazy)))
       (#t(:sort <>) . #t(:op #^<>)) (#t(:sort ==) . #t(:op #^==))
       (#t(:sort <<) . #t(:op #^<<))
       (#t(:sort adt-or-theories) . #t(:op #^adt-or-modules))
       (#t(:sort expname-opt-0)
        . #t(:union (:op #^expformula) (:op #^exptype)
             (:sort type-expr) (:op #^noexpkind)))
       (#t(:sort adtcases) . #t(:op #^adtcases))
       (#t(:sort fun-array)
        . #t(:union (:op #^array) (:op #^function)))
       (#t(:sort -) . #t(:op #^-)) (#t(:sort <=>) . #t(:op #^<=>))
       (#t(:sort lib-decl) . #t(:op #^lib-decl))
       (#t(:sort coind-decl) . #t(:op #^coind-decl))
       (#t(:sort judgement-opt-0)
        . #t(:union (:op #^rec-subtype-of) (:op #^rec-has-type)
             (:op #^subtype-of) (:op #^has-type)))
       (#t(:sort simplebind) . #t(:op #^simplebind))
       (#t(:sort type-def) . #t(:op #^type-def))
       (#t(:sort typed-ids) . #t(:op #^typed-ids))
       (#t(:sort type-keyword)
        . #t(:union (:op #^typeplus) (:op #^nonempty-type)
             (:op #^type)))
       (#t(:sort selection) . #t(:op #^selection))
       (#t(:sort unique-name-opt-0)
        . #t(:union (:op #^formula) (:op #^typed) (:op #^type)
             (:op #^noqual)))
       (#t(:sort bind-alt-0)
        . #t(:union (:op #^binds) (:sort simplebind))))))

(lang:lang-define 
:name "pvs"
:conc-name "pvs"
:code-package "PVS"
:abs-syn-package "PVS"
:use-packages '("NEWATTR" "LANG" "SB-RUNTIME" "SORT" "TERM" "OCC" "OPER" "ERGOLISP")
:sub-languages '("LEXICAL-TERMINALS")
:unparse-nts 't
:parse-routine-name 'PVS::PVS-PARSE
:unparse-routine-name 'PVS::PVS-UNPARSE
:win-unparse-routine-name 'PVS::PVS-WIN-UNPARSE
:sort-table-name 'PVS::*PVS-SORT-TABLE*
:opsig-table-name 'PVS::*PVS-OPSIG-TABLE*
)

